<?php
class ControllerExtensionModuleRoutee extends Controller
{
    /**
     * @var array
     */
    private $error = array();

    /**
     * @return void
     */
    public function index()
    {
        $this->load->language('extension/module/routee');
        $this->document->setTitle($this->language->get('heading_title'));
        $this->load->model('setting/setting');

        if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
            $resultJson = $this->getAuthenticate($this->request->post);

            if (!isset($resultJson)) {
                $this->session->data['warning'] = $this->language->get('text_wrong_creds');
            } else {
                if (isset($resultJson['message']) && ($resultJson['message'] == "Authorization succeed")) {
                    $module_routee_uuid = $this->config->get('module_routee_uuid');
                    if ($module_routee_uuid == "") {
                        $this->request->post['module_routee_uuid'] = $resultJson['uuid'];
                    }
                    $this->mergeDefaultExportValues();
                    $this->model_setting_setting->editSetting('module_routee', $this->request->post);
                    $this->hitEuApi(2, $resultJson['uuid']);
                    $this->session->data['success'] = $this->language->get('text_success');
                } else {
                    $this->session->data['warning'] = $this->language->get('text_wrong_creds');
                }
            }
            $this->response->redirect($this->url->link('extension/module/routee', 'user_token=' . $this->session->data['user_token'], true));
        }
        $array = array(
            'success',
            'warning'
        );
        foreach ($array as $value) {
            $data[$value] = $this->error[$value] ?? '';

            if (isset($this->session->data[$value])) {
                $data[$value] = $this->session->data[$value];
                unset($this->session->data[$value]);
            } else {
                $data[$value] = '';
            }
        }


        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');
        $data['breadcrumbs'] = $this->setBreadcrumbs();

        $data['action'] = $this->url->link('extension/module/routee', 'user_token=' . $this->session->data['user_token'], true);
        $data['cancel'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=module', true);

        $data['error_username'] = $this->error['username'] ?? '';
        $data['error_password'] = $this->error['password'] ?? '';

        $values = array(
            'module_routee_uuid',
            'module_routee_status',
            'module_routee_username',
            'module_routee_password'
        );
        foreach ($values as $value) {
            $data[$value] = $this->getValue($value);
        }

        $this->response->setOutput($this->load->view('extension/module/routee', $data));
    }

    public function getValue($param)
    {
        return $this->request->post[$param] ?? $this->config->get($param);
    }

    public function install()
    {
        $this->load->library('routeeurls');
        $this->load->model('setting/setting');
        $this->load->model('user/user');

        $this->routeeurls->init(1, [], '', $this->model_user_user, $this->model_setting_setting);
        $this->createLogsTable();
    }

    /**
     * @return void
     */
    public function uninstall()
    {
        try {
            $this->sendLogs('event triggered', 'event', 'Uninstall');
            $this->hitEuApi(3);
            if ($this->config->get('module_routee_uuid') != "") {
                $customerCartValue = array();

                $customerCartValue['data'] = array();
                $customerCartValue["uuid"] = $this->config->get('module_routee_uuid');
                $customerCartValue["event"] = 'Uninstall';

                $this->sendLogs('event before api call', 'events', 'Uninstall');
                $result = $this->routee->curl($customerCartValue, 'uninstall');
                $this->sendLogs('event after api call', 'event', 'Uninstall');
                $this->routee->write("Uninstall: " . json_encode($customerCartValue) . " Result: " . $result);
            }
            $this->dropLogsTable();

            $this->load->model('setting/setting');
            $this->model_setting_setting->deleteSetting('module_routee');

        } catch (Exception $exception) {
            $this->sendLogs($exception->getMessage(), 'events', 'Uninstall', $exception);
        }
    }

    public function createLogsTable()
    {
        try {
            $tableName = DB_PREFIX . 'routee_shop_logs';
            $this->db->query("CREATE TABLE IF NOT EXISTS " . $tableName . " (
                                  `id` int(11) NOT NULL AUTO_INCREMENT,
                                  `store_url` varchar(255) NOT NULL,
                                  `log_type` varchar(255) NOT NULL,
                                  `event_type` varchar(255) NOT NULL,
                                  `is_exported` enum('0','1') NOT NULL,
                                  `created_at` varchar(255) NOT NULL,
                                  `updated_at` datetime NOT NULL,
                                  `log_data` longtext NOT NULL,
                                  PRIMARY KEY (id)
                                ) ENGINE=MyISAM DEFAULT CHARSET=latin1");
        } catch (Exception $exception) {
        }
    }

    public function mergeDefaultExportValues()
    {
        $values = [
            'module_routee_product_exported' => 0,
            'module_routee_customer_exported' => 0,
            'module_routee_order_exported' => 0,
            'module_routee_wishlist_exported' => 0,
            'module_routee_newsletter_exported' => 0
        ];
        $this->request->post = array_merge($this->request->post, $values);
    }

    public function dropLogsTable()
    {
        $tableName = DB_PREFIX . 'routee_shop_logs';
        $this->db->query("DROP TABLE IF EXISTS " . $tableName);
    }

    /**
     * @return bool
     */
    protected function validate()
    {
        if (!$this->user->hasPermission('modify', 'extension/module/routee')) {
            $this->error['warning'] = $this->language->get('error_permission');
        }

        if (!$this->request->post['module_routee_username']) {
            $this->error['username'] = $this->language->get('error_username');
        }

        if (!$this->request->post['module_routee_password']) {
            $this->error['password'] = $this->language->get('error_password');
        }

        return !$this->error;
    }

    /**
     * @param $post
     * @return mixed|void
     */
    public function getAuthenticate($post)
    {

        $this->load->model('setting/modification');
        $this->load->library('routee');
        $plugin_data = $this->model_setting_modification->getModificationByCode('Routee API');

        try {
            $this->sendLogs('event triggered', 'auth', 'Auth');
            $postData = [
                "username" => $post['module_routee_username'],
                "password" => $post['module_routee_password'],
                "source" => "Opencart",
                "type" => "E-Commerce",
                "version" => VERSION,
                "plugin_version" => $plugin_data['version'],
                "domain" => HTTP_CATALOG
            ];

            $this->sendLogs('event before api call', 'auth', 'Auth');
            $result = $this->routee->curl($postData, 'auth');
            $this->sendLogs('event after api call', 'auth', 'Auth');
            return $result;
        } catch (Exception $exception) {
            $this->sendLogs($exception->getMessage(), 'auth', 'Auth', $exception);
        }
    }

    /**
     * @param $message
     * @param $event
     * @param $logType
     * @param $excep
     * @return void
     */
    public function sendLogs($message, $event, $logType, $excep = null)
    {
        try {
            $this->load->library('routeelogs');
            $this->routeelogs->saveLogs($logType, $event, $message, $excep);
        } catch (Exception $exception) {
        }
    }

    /**
     * @return array[]
     */
    public function getRouteeUrl($type) {
        $this->load->library('routeeurls');
        return $this->routeeurls->getDynamicUrl($type);
    }

    /**
     * @param $type
     * @param $uuid
     * @return void
     */
    public function hitEuApi($type, $uuid = '') {
        $this->load->model('setting/setting');
        $this->load->model('user/user');
        $this->load->library('routeeurls');
        $this->routeeurls->init($type, [], $uuid, $this->model_user_user, $this->model_setting_setting);
    }

    /**
     * @return array[]
     */
    public function setBreadcrumbs() {
        return array(
            array(
                'text' => $this->language->get('text_home'),
                'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true)
            ),
            array(
                'text' => $this->language->get('text_extension'),
                'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=module', true)
            ),
            array(
                'text' => $this->language->get('heading_title'),
                'href' => $this->url->link('extension/module/routee', 'user_token=' . $this->session->data['user_token'], true)
            )
        );
    }
}
