<?php
class ControllerExtensionModuleRouteeProducts extends Controller {
    /**
     * @return void
     */
    public function massExport() {
        
    	$this->load->library('routee');
        $action = $_POST['action'];
        $uuid = $this->config->get('module_routee_uuid');
        $this->load->language('extension/module/routee/products');
        $reload = 0;
        if (empty($uuid)) {
            $this->session->data['error_warning'] = $this->language->get('text_auth_error');
            $result = array(
                'reload' => 1
            );
        } else {
            $limit = $this->routee->getRPRLimit();
            $this->load->model('catalog/product');
            $this->load->model('catalog/category');
            $this->load->model('setting/setting');
            $filter_data = array(
                'sort'    			=> 'pd.name',
                'order'   			=> 'ASC'
            );
            if ($action == "get_count") {
                $productData = $this->model_catalog_product->getProducts($filter_data);
                $t_count = count($productData);
                if ($t_count == 0){
                    $this->session->data['not_exists'] = $this->language->get('text_not_exists');
                    $this->model_setting_setting->editSettingValue('module_routee', 'module_routee_product_exported', 1 );
                    $result = array(
                        'reload' => 1
                    );
                } else {
                    $result = array(
                        'reload' => $reload,
                        'text' => sprintf($this->language->get('text_process'), $t_count)
                    );
                }
            } elseif ( $action == "export_data" ) {
                unset($this->session->data['error_warning'] , $this->session->data['success']);
                $cycle = $_POST['cycle_count'];
                $filter_data['limit'] = $limit;
                $filter_data['start'] = ($cycle - 1) * $limit;
                $productInfo = [];
                $productData = $this->model_catalog_product->getProducts($filter_data);
                $c_count = count($productData);

                foreach($productData as $k => $productRow) {
                    $searchProduct = $category = $discount = array();
                    $special = false;
                    $product_specials = $this->model_catalog_product->getProductSpecials($productRow['product_id']);

                    foreach ($product_specials  as $product_special) {
                        if (($product_special['date_start'] == '0000-00-00' || strtotime($product_special['date_start']) < time()) && ($product_special['date_end'] == '0000-00-00' || strtotime($product_special['date_end']) > time())) {
                            $special = $product_special['price'];
                            break;
                        }
                    }

                    $categories = $this->model_catalog_product->getProductCategories($productRow['product_id']);
                    foreach ($categories as $category_id) {
                        $category_info = $this->model_catalog_category->getCategory($category_id);
                        if ($category_info) {
                            $category[] = $category_info['name'];
                        }
                    }

                    $image_link = "";
                    if (isset($productRow['image'])) {
                        $image_link = HTTP_CATALOG.$productRow['image'];
                    }

                    $product_link = HTTP_CATALOG.'index.php?route=product/product&product_id='.$productRow['product_id'];
                    $product_discounts = $this->model_catalog_product->getProductDiscounts($productRow['product_id']);
                    foreach ($product_discounts as $product_discount) {
                        $discount[] = array(
                            'price'       => $this->tax->calculate($product_discount['price'], $productRow['tax_class_id'], $this->config->get('config_tax')),
                            'quantity'    => $product_discount['quantity'],
                            'date_start'  => ($product_discount['date_start'] != '0000-00-00') ? $product_discount['date_start'] : '',
                            'date_end'    => ($product_discount['date_end'] != '0000-00-00') ? $product_discount['date_end'] : ''
                        );
                    }

                    $product_description = $this->model_catalog_product->getProductDescriptions($productRow['product_id']);
                    $price = ($special) ? $special : $productRow['price'];

                    foreach ($product_description as $language_id => $value) {
                        $searchProduct = array(
                            'product_id' => $productRow['product_id'],
                            'name' => $value['name'],
                            'description' => html_entity_decode($value['description'], ENT_QUOTES, 'UTF-8'),
                            'categories' => $category,
                            'stock_quantity' => $productRow['quantity'],
                            'price' => $this->tax->calculate($price, $productRow['tax_class_id'], $this->config->get('config_tax')),
                            'discount' => $discount,
                            'image_link' => $image_link,
                            'product_link' => $product_link
                        );
                    }
                    $productInfo[] = $searchProduct;
                }
                $resultJson = $this->sendData($uuid, $productInfo);

                if (!isset($resultJson)){
                    $this->session->data['error_warning'] = $this->language->get('text_error');
                    $reload = 1;
                } elseif (!empty($resultJson['message'])) {
                    if ($c_count < $limit) {
                        $this->model_setting_setting->editSettingValue('module_routee', 'module_routee_product_exported', 1 );
                        $this->session->data['success'] = $this->language->get('text_success');
                        $reload = 1;
                    }
                }
                $result = array(
                    'c_count' => $c_count,
                    'reload' => $reload
                );
            }
        }

        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($result));
    }

    /**
     * @param $uuid
     * @param $productInfo
     * @return mixed|void
     */
    public function sendData($uuid, $productInfo) {
        try {
            $this->load->library('routee');
            $this->sendLogs('event triggered');
            $data["uuid"] = $uuid;
            $data['data'][0] = array(
                'name' => 'Products',
                'description' => 'Shop Products',
                'object' => $productInfo,
            );
            $this->sendLogs('event before api call');
            $result = $this->routee->curl($data, 'data');
            $this->sendLogs('event after api call');
            $log = is_array($result) ? json_encode($result) : $result;
            $this->routee->write("Mass Products: ".json_encode($data)." Result: ".$log);

            return $result;
        } catch (Exception $exception) {
            $this->sendLogs($exception->getMessage(), $exception);
        }
    }

    /**
     * @param $message
     * @param $excep
     * @return void
     */
    public function sendLogs($message, $excep=null) {
        try {
            $this->load->library('routeelogs');
            $this->routeelogs->saveLogs('ProductExport','massdata', $message, $excep);
        } catch (Exception $exception) {}
    }
}