<?php
/**
 * RouteeUrls class
 */
class RouteeUrls {

    /**
     * @var
     */
    private $db;

    /**
     * @var
     */
    private $config;

    /**
     * @var
     */
    private $session;

    /**
     * @var
     */
    private $user;

    /**
     * @var
     */
    private $userModel;

    /**
     * @var
     */
    private $settingModel;

    /**
     * @var
     */
    private $uuid;

    /**
     * @var string
     */
    private $dynamicEndpoint = 'https://eu.api.wm.amdtelecom.net/api/endpoints';

    /**
     * @var string[]
     */
    private $urlTypes = [
        'auth',
        'data',
        'event',
        'logs'
    ];

    /**
     * Constructor
     *
     * @param $registry
     */
    public function __construct($registry) {
        $this->config = $registry->get('config');
        $this->session = $registry->get('session');
        $this->db = $registry->get('db');
    }

    /**
     * @param $type { 1 = fetch data, 2 = save callback, 3 = uninstall, 4 hit from EU API }
     * @param array $postedData
     * @param string $uuid
     * @param null $userModel
     * @param null $settingModel
     * @return void
     */
    public function init($type, $postedData = [], $uuid = '', $userModel = null, $settingModel = null)
    {
        $this->userModel = $userModel;
        $this->settingModel = $settingModel;
        $this->uuid = $uuid;

        if ($type != 4){
            $userId = $this->session->data['user_id'];
            $userInfo = $this->userModel->getUser($userId);
            $this->user = $userInfo;
        }

        switch ($type){
            case 1:
                $this->fetch();
                break;
            case 2:
                $this->saveCallback();
                break;
            case 3:
                $this->uninstall();
                break;
            case 4:
                $this->getDataFromCallback($postedData);
                break;
            default:
                return;
        }
    }

    /**
     * @param $url
     * @param $type
     * @return void
     */
    public function saveUrl($url, $type)
    {
        try {
            $data['module_routee_'.$type] = $url;
            !empty($this->settingModel) ?
            $this->settingModel->editSetting('module_routee_'.$type, $data) :
            $this->editSetting('module_routee_'.$type, $data);
        }catch (Exception $exception){
        }
    }

    /**
     * @param $postData
     * @param $uri
     * @return void
     */
    public function getAndSaveUrls($postData, $uri)
    {
        $urls = $this->makeHttpCall($postData, $uri);
        $this->compileAndSave($urls);
    }

    /**
     * @param $urls
     * @return void
     */
    public function compileAndSave($urls)
    {
        if (!empty($urls)){
            foreach ($urls as $key => $url) {
                $this->saveUrl($url['url'], $url['type']);
            }
        }
    }

    /**
     * @param $type
     * @return null
     */
    public function getDynamicUrl($type)
    {
        try {
           return $this->config->get('module_routee_'.$type);
        }catch (Exception $exception){
            return null;
        }
    }

    /**
     * @param bool $withAuth
     * @return array
     */
    public function commonEndpointPayload($withAuth = false)
    {
        $response = [
            "store_url" => HTTP_CATALOG,
            "platform" => "Opencart3",
            "plugin_version" => "1.5.2",
            "platform_version" => VERSION,
            'user_id' => $this->user['user_id'] ?? '',
            "email" => $this->user["email"] ?? '',
        ];

        if ($withAuth){
            $response['store_uuid'] = $this->uuid;
            $response['store_callback_url'] = HTTP_CATALOG. 'index.php?route=common/home/routeecallback';
        }

        return $response;
    }

    /**
     * @return void
     */
    public function uninstall()
    {

        $postData = [
            'store_url' => HTTP_CATALOG
        ];
        $this->makeHttpCall($postData, '/uninstall');
        foreach ($this->urlTypes as $urlType) {
            $this->settingModel->deleteSetting('module_routee_'.$urlType);
        }
    }

    /**
     * @return void
     */
    public function fetch()
    {
        $auth = $this->getDynamicUrl('auth');
        if (empty($auth)){
            $postedData = $this->commonEndpointPayload();
            $this->getAndSaveUrls($postedData, '/get');
        }
    }

    /**
     * @param $code
     * @param $data
     * @param $store_id
     * @return void
     */
    public function editSetting($code, $data, $store_id = 0) {
        $this->db->query("DELETE FROM `" . DB_PREFIX . "setting` WHERE store_id = '" . (int)$store_id . "' AND `code` = '" . $this->db->escape($code) . "'");

        foreach ($data as $key => $value) {
            if (substr($key, 0, strlen($code)) == $code) {
                if (!is_array($value)) {
                    $this->db->query("INSERT INTO " . DB_PREFIX . "setting SET store_id = '" . (int)$store_id . "', `code` = '" . $this->db->escape($code) . "', `key` = '" . $this->db->escape($key) . "', `value` = '" . $this->db->escape($value) . "'");
                } else {
                    $this->db->query("INSERT INTO " . DB_PREFIX . "setting SET store_id = '" . (int)$store_id . "', `code` = '" . $this->db->escape($code) . "', `key` = '" . $this->db->escape($key) . "', `value` = '" . $this->db->escape(json_encode($value, true)) . "', serialized = '1'");
                }
            }
        }
    }

    /**
     * @return void
     */
    public function saveCallback()
    {
        $postedData = $this->commonEndpointPayload(true);
        $this->getAndSaveUrls($postedData, '/save_callback');
    }

    /**
     * @param $postedData
     * @return void
     */
    public function getDataFromCallback($postedData)
    {
        if (!empty($postedData['uuid']) && $postedData['uuid'] == $this->config->get('module_routee_uuid')){
            $urls = !empty($postedData['data']) ? $postedData['data'] : [];
            $this->compileAndSave($urls);
        }
    }

    /**
     * @param $postData
     * @param $uri
     * @return null
     */
    public function makeHttpCall($postData, $uri)
    {
        $postData = http_build_query($postData);
        $url = $this->dynamicEndpoint . $uri;
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($ch);
        if (!empty($result)) {
            $decoded = json_decode($result, true);
            return !empty($decoded['data']) ? $decoded['data'] : [];
        }

        return [];
    }

}
